
function createDiamond(ar, x, y, minrange, maxrange){
	for(var i=minrange; i<=maxrange; i++){
		for(var c=1; c<=i; c++)
			add(ar, [x-i+c, y+c]);
		for(var c=0; c<=i; c++)
			add(ar, [x-i+c, y-c]);
		for(var c=1; c<i; c++)
			add(ar, [x+i-c, y+c]);
		for(var c=0; c<i; c++)
			add(ar, [x+i-c, y-c]);
	}
}

function createEmptyDiamond(ar, x, y, minrange, maxrange){
	for(var i=minrange; i<=maxrange; i++){
		for(var c=1; c<=i; c++)
			if(cellEmpty(x-i+c, y+c)) add(ar, [x-i+c, y+c]);
		for(var c=0; c<=i; c++)
			if(cellEmpty(x-i+c, y-c))  add(ar, [x-i+c, y-c]);
		for(var c=1; c<i; c++)
			if(cellEmpty(x+i-c, y+c)) add(ar, [x+i-c, y+c]);
		for(var c=0; c<i; c++)
			if(cellEmpty(x+i-c, y-c)) add(ar, [x+i-c, y-c]);
	}
}

function createLinear(ar, x, y, minrange, maxrange, passthrough){
	var bl = false, br = false, bb = false, bt = false;
	for(var i=minrange; i<=maxrange; i++){
		if(!bt) add(ar, [x, y+i]); if(!bb) add(ar, [x, y-i]);
		if(!br) add(ar, [x+i, y]); if(!bl) add(ar, [x-i, y]);
		if(!passthrough){
			if(y+i < worldY && worldGrid[(y+i)*worldX+x] != null) bt = true;
			if(y-i >= 0 && worldGrid[(y-i)*worldX+x] != null) bb = true;
			if(x+i < worldX && worldGrid[y*worldX+x+i] != null) br = true;
			if(x-i >= 0 && worldGrid[y*worldX+x-i] != null) bl = true;
		}
	}
}

function forEventsInRange(x, y, minrange, maxrange, func){
	for(var i=0; i<eventList.length; i++){
		var e = eventList[i];
		var m_dist = dist(getX(e), getY(e), x, y);
		if((m_dist >= minrange) & (m_dist <= maxrange)) func(e);
	}
}

var modifiers = [[1,1,1,1,1,1,1,1,1,1],[0.9,0.7,0.8,1,1,0.9,1,1,1,1],[0.6,0.7,0.7,1,1,0.6,1,1,1,1],[0.7,1,0.9,0.3,2,0.7,0.5,1,1,1],
[0.8,0.8,0.8,0.3,0.6,0.7,1,1,1,1],[0.5,0.5,0.5,1,1.3,0.5,1.3,1,1,1],[0.6,0.6,0.6,1,1,0.6,1,1,1,1],[0.1,0.1,0.1,1.3,0.5,0.1,0.5,1,3,3],
[0.7,0.5,0.3,1.3,1.3,0.7,0.4,1,4,0.1],[0.6,0.7,0.7,0.8,0.8,0.6,0.8,0.8,0,0.2],[0.6,0.7,0.7,0.8,0.8,0.6,0.8,0.8,0.2,0]];

function getModifier(def_attr, atk_attr){return modifiers[def_attr][atk_attr];}

function getDefAttrName(i){switch(i){
	case DEF_LIGHT: return "Light"; case DEF_MEDIUM: return "Medium"; case DEF_HEAVY: return "Heavy"; case DEF_FIRE: return "Fire";
	case DEF_WATER: return "Water"; case DEF_EARTH: return "Earth"; case DEF_AIR: return "Air"; case DEF_GHOST: return "Ghost";
	case DEF_UNDEAD: return "Undead"; case DEF_HOLY: return "Holy"; case DEF_EVIL: return "Evil";
} return "None";}

function createWeapon(name, icon, descr, type, complexity){
	var weapon = new Object();
	weapon.name = name;
	weapon.icon = icon;
	weapon.description = descr;
	weapon.mindamage = 0; weapon.maxdamage = 0; weapon.minrange = 0; weapon.maxrange = 0;
	weapon.minaoerange = 0; weapon.maxaoerange = 0; weapon.apcost = 0;
	weapon.getRange = function(ar, x, y){};
	weapon.inRange = function(x, y, cx, cy){return false;};
	weapon.getAoE = function(ar, x, y){};
	weapon.forEventsInAoE = function(x, y, func){};
	weapon.critical = 0;
	weapon.complexity = complexity;
	weapon.type = type;
	weapon.animation = function(x, y){if(this.sound) playSE(this.sound); if(animationCallback) animationCallback();}
	weapon.getAvg = function(stats){
		return this.mindamage+Math.floor(Math.pow(0.45, (this.complexity/stats[this.type]))*(this.maxdamage-this.mindamage)+0.5);
	};
	weapon.getMin = function(stats){
			return this.mindamage+Math.floor(Math.pow(0.1, (this.complexity/stats[this.type]))*(this.maxdamage-this.mindamage)+0.5);
	}
	weapon.getMax = function(stats){
		return this.mindamage+Math.floor(Math.pow(0.8, (this.complexity/stats[this.type]))*(this.maxdamage-this.mindamage)+0.5);
	}
	weapon.baseDamage = function(stats, attribute, x, y, color){
		var mod = 1;
		if(typeof(attribute) != 'undefined' && typeof(this.attribute) != 'undefined') mod = getModifier(attribute, this.attribute);
		var crit = false;
		var r = Math.pow(Math.random()*0.7+0.1, (this.complexity/stats[this.type]));
		var dmg = Math.floor((this.mindamage+r*(this.maxdamage-this.mindamage))*mod+0.5);
		if((this.critical > 0) & (r < 0.01)) dmg = 0;
		if(Math.random() < this.critical){dmg *= 2; crit = true;}
		stats[this.type] += r*0.2*this.complexity/stats[this.type];
		if(color)
			showFloatText("<font style='color:"+color+";'>"+((crit)?"Crit!<br>":"")+((dmg == 0)?"Miss":dmg)+"</font>", x, y);
		return dmg;
	};
	weapon.forSpacesInRange = function(x, y, func){
		var tmp = new Array(); this.getRange(tmp, x, y);
		for(var i=0; i<tmp.length; i++) func(tmp[i][0], tmp[i][1]);
	}
	weapon.use = function(){equip(this, WEAPON); return true;}
	return weapon;
}

function setDiamondRange(weapon, minrange, maxrange){
	weapon.minrange = minrange; weapon.maxrange = maxrange;
	weapon.getRange = function(ar, x, y){createDiamond(ar, x, y, this.minrange, this.maxrange);};
	weapon.inRange = function(x, y, cx, cy){var m_dist = dist(x, y, cx, cy); return (m_dist >= this.minrange) & (m_dist <= this.maxrange);};
}

function setLinearRange(weapon, minrange, maxrange, passthrough){
	weapon.minrange = minrange; weapon.maxrange = maxrange;
	weapon.passthrough = passthrough;
	weapon.getRange = function(ar, x, y){createLinear(ar, x, y, this.minrange, this.maxrange, this.passthrough);};
	weapon.inRange = function(x, y, cx, cy){
		var m_dist = dist(x, y, cx, cy);
		if((m_dist < this.minrange) | (m_dist > this.maxrange) | ((x != cx) & (y != cy))) return false;
		if(this.passthrough) return true;
		if(x == cx){
			if(y < cy){for(var i=y+1; i<Math.min(worldY, cy); i++) if(worldGrid[i*worldX+x] != null) return false;}
			else{for(var i=y-1; i>Math.max(0, cy); i--) if(worldGrid[i*worldX+x] != null) return false;}
			return true;
		}
		if(x < cx){for(var i=x+1; i<Math.min(worldX, cx); i++) if(worldGrid[y*worldX+i] != null) return false;}
		else{for(var i=x-1; i>Math.max(0, cx); i--) if(worldGrid[y*worldX+i] != null) return false;}
		return true;
	};
}

function setEmptyRange(weapon, minrange, maxrange){
	weapon.minrange = minrange; weapon.maxrange = maxrange;
	weapon.getRange = function(ar, x, y){createEmptyDiamond(ar, x, y, this.minrange, this.maxrange);};
	weapon.inRange = function(x, y, cx, cy){var m_dist = dist(x, y, cx, cy); return (m_dist >= this.minrange) & (m_dist <= this.maxrange) & cellEmpty(x, y);};
}


function setDiamondAoE(weapon, minrange, maxrange){
	weapon.minaoerange = minrange; weapon.maxaoerange = maxrange;
	weapon.getAoE = function(ar, x, y){createDiamond(ar, x, y, this.minaoerange, this.maxaoerange);};
	weapon.forEventsInAoE = function(x, y, func){forEventsInRange(x, y, this.minaoerange, this.maxaoerange, func);};
}

function createMeleeWeapon(name, id, descr, anim, mindmg, maxdmg, crit, apcost, complexity, cost){
	var weapon = createWeapon(name, id, descr, MELEE, complexity);
	setDiamondRange(weapon, 1, 1);
	setDiamondAoE(weapon, 0, 0);
	weapon.getReference = function(){return "createMeleeWeapon(\""+name+"\","+id+",\""+descr+"\",\""+anim+"\","+mindmg+","+maxdmg+","+crit+","+apcost+","+complexity+","+cost+")";};
	weapon.mindamage = mindmg;
	weapon.maxdamage = maxdmg;
	weapon.critical = crit;
	weapon.apcost = apcost;
	weapon.attribute = (id < 4)?ATK_SLASH:(id < 6)?ATK_PIERCE:(id < 12)?ATK_SLASH:(id == 21)?ATK_AIR:(id < 25)?ATK_BLUNT:ATK_SLASH;
	weapon.market = (cost >= 1000)?FINE_WEAPON_MARKET:CHEAP_WEAPON_MARKET;
	if((id != 6) & (id != 20) & (id != 21) & (id != 26)) weapon.onehanded = true;
	weapon.cost = cost;
	weapon.animation = eval(anim);
	return weapon;
}

function createSpearWeapon(name, id, descr, anim, mindmg, maxdmg, crit, apcost, complexity, cost, range){
	var weapon = createMeleeWeapon(name, id, descr, anim, mindmg, maxdmg, crit, apcost, complexity, cost);
	weapon.onehanded = false;
	weapon.getReference = function(){return "createSpearWeapon(\""+name+"\","+id+",\""+descr+"\",\""+anim+"\","+mindmg+","+maxdmg+","+crit+","+apcost+","+complexity+","+cost+","+range+")";};
	setLinearRange(weapon, 1, range, false);
	setDiamondAoE(weapon, 0, 0);
	return weapon;
}

function createRangedWeapon(name, id, descr, anim, mindmg, maxdmg, crit, apcost, complexity, cost, range){
	var weapon = createMeleeWeapon(name, id, descr, anim, mindmg, maxdmg, crit, apcost, complexity, cost);
	weapon.onehanded = false;
	weapon.getReference = function(){return "createRangedWeapon(\""+name+"\","+id+",\""+descr+"\",\""+anim+"\","+mindmg+","+maxdmg+","+crit+","+apcost+","+complexity+","+cost+","+range+")";};
	setDiamondRange(weapon, 1, range);
	setDiamondAoE(weapon, 0, 0);
	weapon.type = RANGED;
	weapon.attribute = ((id == 22) | (id == 23))?ATK_BLUNT:ATK_PIERCE;
	return weapon;
}

var barehanded = createMeleeWeapon("", 0, "", "punchAnim", 1, 8, 0.005, 8, 15, 0);
barehanded.getReference = function(){return "barehanded";}
barehanded.attribute = ATK_BLUNT;